@@ -16,11 +16,11 @@ use lib "$FindBin::Bin/../lib";
 
 use Protocol::DBus::Client;
 
-my $dbus = Protocol::DBus::Client::system();
+my $dbus = $> ? Protocol::DBus::Client::login_session() : Protocol::DBus::Client::system();
 
 # $dbus->big_endian(1);
 
-$dbus->do_authn();
+$dbus->initialize();
 
 my $signal_name = 'ProtocolDBusFDPass';
 
@@ -37,12 +37,12 @@ $dbus->send_call(
 
 $dbus->get_message();
 
-my $recv_name = $dbus->get_connection_name();
+my $recv_name = $dbus->get_unique_bus_name();
 
 my $pid = fork or do {
-    my $dbus = Protocol::DBus::Client::system();
+    my $dbus = my $dbus = $> ? Protocol::DBus::Client::login_session() ? Protocol::DBus::Client::system();
 
-    $dbus->do_authn();
+    $dbus->initialize();
 
     pipe( my $r, my $w );
 
