@@ -3,7 +3,7 @@ package Protocol::DBus;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =encoding utf8
 
@@ -27,8 +27,7 @@ For blocking I/O:
         destination => 'org.freedesktop.DBus',
         signature => 's',
         body => [ 'org.freedesktop.DBus' ],
-        on_return => sub { my ($msg) = @_ },
-    );
+    )->then( sub { my $msg = shift; ..  } );
 
     my $msg = $dbus->get_message();
 
@@ -72,25 +71,30 @@ For non-blocking I/O:
 
 =head1 DESCRIPTION
 
-This is an original, pure-Perl implementation of client logic for
+This is an original, pure-Perl implementation of client messaging logic for
 L<the D-Bus protocol|https://dbus.freedesktop.org/doc/dbus-specification.html>.
 
 It’s not much more than an implementation of the wire protocol; it doesn’t
 know about objects, services, or anything else besides the actual messages.
-That said, what’s here already should allow implementation of anything you
-can do with D-Bus; moreover, it would not be difficult to implement
-convenience logic—e.g., to mimic interfaces like L<Net::DBus>—on top of
-what is here now.
+This is fine, of course, if all you want to do is, e.g., replace
+an invocation of C<gdbus> or C<dbus-send> with pure Perl.
 
-Right now this distribution is an experimental effort. If you use it in your
-project, be sure to check the changelog before deploying a new version. Please
-file bug reports as appropriate.
+If you want an interface that mimics D-Bus’s actual object system,
+you’ll need to implement it yourself or to look elsewhere.
+(See L</SEE ALSO> below.)
 
-See L<Protocol::DBus::Client> and the above sample for a starting point.
+=head1 STATUS
+
+This project is in BETA status. While the API should be pretty stable now,
+breaking changes can still happen. If you use this module
+in your project, you B<MUST> check the changelog before deploying a new
+version. Please file bug reports as appropriate.
 
 =head1 EXAMPLES
 
-See the distribution’s F<examples/> directory.
+See L<Protocol::DBus::Client> and the above sample for a starting point.
+
+Also see the distribution’s F<examples/> directory.
 
 =head1 NOTES
 
@@ -99,6 +103,10 @@ See the distribution’s F<examples/> directory.
 =item * UNIX FD support requires that L<Socket::MsgHdr> be loaded at
 authentication time.
 
+=item * Certain OSes may require L<Socket::MsgHdr> to function.
+(Linux, notably, does not.) It depends if your OS can send local socket
+credentials without recourse to C<sendmsg(2)>.
+
 =item * EXTERNAL and DBUS_COOKIE_SHA1 authentication is supported.
 
 =back
@@ -107,8 +115,6 @@ authentication time.
 
 =over
 
-=item * Add conveniences like match rule logic.
-
 =item * Improve parsing of bus paths in environment variables.
 
 =item * Add more tests.
@@ -117,8 +123,8 @@ authentication time.
 
 =head1 SEE ALSO
 
-L<Net::DBus> uses libdbus (via XS) as its backend. It’s more mature and
-more idiomatic as to how a D-Bus application is normally written, but
-it’s also heavier, and it doesn’t appear to support passing filehandles.
+The most mature, stable D-Bus implementation in Perl is L<Net::DBus>,
+an XS binding to L<libdbus|https://www.freedesktop.org/wiki/Software/dbus/>,
+the reference D-Bus implementation.
 
 =cut
