@@ -18,8 +18,7 @@ Protocol::DBus::Peer - base class for a D-Bus peer
         path => '/org/freedesktop/DBus',
         destination => 'org.freedesktop.DBus',
         body => [ 'org.freedesktop.DBus' ],
-        on_return => sub { my ($msg) = @_ },
-    );
+    )->then( sub { .. } );
 
     my $msg = $dbus->get_message();
 
@@ -42,6 +41,7 @@ implementation.)
 =cut
 
 use Call::Context;
+use Promise::ES6;
 
 use Protocol::DBus::Message;
 use Protocol::DBus::Parser;
@@ -55,7 +55,7 @@ use Protocol::DBus::WriteMsg;
 
 This returns a single instace of L<Protocol::DBus::Message>, or undef if
 no message is available. It will also fire the appropriate “on_return”
-method on METHOD_RETURN messages.
+method on METHOD_RETURN or ERROR messages.
 
 The backend I/O logic reads data in chunks; thus, if there is a message
 already available in the read buffer, no I/O is done. If you’re doing
@@ -98,48 +98,118 @@ sub flush_write_queue {
 
 =head2 I<OBJ>->send_call( %OPTS )
 
-Send a METHOD_CALL message to the server.
+Send a METHOD_CALL message.
 
 %OPTS are C<path>, C<interface>, C<member>, C<destination>, C<signature>,
-C<body>, and C<on_return>. These do as you’d expect, with the following
-caveats:
+and C<body>. These do as you’d expect, but note that C<body>, if given,
+must be an array reference.
 
-=over
+The return value is an instance of L<Promise::ES6> that will resolve when
+a METHOD_RETURN arrives in response, or reject when an ERROR arrives. The
+promise both resolves and rejects with a L<Protocol::DBus::Message> instance
+that represents the response.
 
-=item * C<body>, if given, must be an array reference. See
-L<Protocol::DBus::Message> for a discussion of how to map between D-Bus and
-Perl.
-
-=item * The C<on_return> callback receives the server’s response
-message (NB: either METHOD_RETURN or ERROR) as argument.
-
-=back
+Note that exceptions can still happen (outside of the promise), e.g., if
+your input is invalid or if there’s a socket I/O error.
 
 =cut
 
+use constant _METHOD_RETURN_NUM => Protocol::DBus::Message::Header::MESSAGE_TYPE()->{'METHOD_RETURN'};
+
 sub send_call {
     my ($self, %opts) = @_;
 
-    my $cb = delete $opts{'on_return'};
-
-    my $ret = $self->_send_msg(
+    $self->_send_msg(
         %opts,
         type => 'METHOD_CALL',
     );
 
-    if ($cb) {
+    # Don’t create a promise if we were called in void context.
+    return defined(wantarray) && do {
         my $serial = $self->{'_last_sent_serial'};
-        $self->{'_on_return'}{$serial} = $cb;
-    }
 
-    return $ret;
+        # Keep references to $self out of the callback
+        # in order to avoid memory leaks.
+        my $on_return_hr = $self->{'_on_return'} ||= {};
+
+        return Promise::ES6->new( sub {
+            my ($resolve, $reject) = @_;
+
+            $on_return_hr->{$serial} = sub {
+                if ($_[0]->get_type() == _METHOD_RETURN_NUM()) {
+                    $resolve->($_[0]);
+                }
+                else {
+                    $reject->($_[0]);
+                }
+            };
+        } );
+    };
 }
 
-sub send_signal {
-    my ($self, %opts) = @_;
+=head2 I<OBJ>->send_return( $ORIG_MSG, %OPTS )
+
+Send a METHOD_RETURN message.
+
+Arguments are similar to C<send_call()> except for the header differences
+that the D-Bus specification describes. Also, C<destination> is not given
+directly but is instead inferred from the $ORIG_MSG. (Behavior is
+undefined if this parameter is given directly.)
+
+=cut
+
+sub send_return {
+    my ($self, $orig_msg, @opts_kv) = @_;
 
     return $self->_send_msg(
-        %opts,
+        _response_fields_from_orig_msg($orig_msg, \@opts_kv),
+        type => 'METHOD_RETURN',
+    );
+}
+
+=head2 I<OBJ>->send_error( $ORIG_MSG, %OPTS )
+
+Like C<send_return()>, but sends an error instead. The
+C<error_name> parameter is required.
+
+=cut
+
+sub send_error {
+    my ($self, $orig_msg, @opts_kv) = @_;
+
+    return $self->_send_msg(
+        _response_fields_from_orig_msg($orig_msg, \@opts_kv),
+        type => 'ERROR',
+    );
+}
+
+sub _response_fields_from_orig_msg {
+
+    return (
+
+        # This has to honor a passed “destination”
+        # so that we can implement a D-Bus server in tests.
+        destination => $_[0]->get_header('SENDER'),
+
+        @{ $_[1] },
+
+        # Reject callers’ attempts to set this one.
+        reply_serial => $_[0]->get_serial(),
+    );
+}
+
+=head2 I<OBJ>->send_signal( %OPTS )
+
+Like C<send_call()> but sends a signal rather than a method call,
+and a promise is not returned.
+
+=cut
+
+sub send_signal {
+    my ($self, @opts_kv) = @_;
+
+    return $self->_send_msg(
+        @opts_kv,
         type => 'SIGNAL',
     );
 }
@@ -151,7 +221,8 @@ sub send_signal {
 Same interface as C<blocking()>, but this sets/gets/toggles whether to send
 big-endian messages instead of little-endian.
 
-(I’m not sure why it would matter?)
+By default this library uses the system’s native byte order, so you probably
+have little need for this function.
 
 =cut
 
@@ -229,6 +300,19 @@ sub pending_send {
 
 #----------------------------------------------------------------------
 
+# undocumented
+sub new {
+    my ($class, $socket) = @_;
+
+    my $self = bless { _socket => $socket }, $class;
+
+    $self->_set_up_peer_io( $socket );
+
+    return $self;
+}
+
+#----------------------------------------------------------------------
+
 sub _set_up_peer_io {
     my ($self, $socket) = @_;
 
@@ -259,7 +343,9 @@ sub _send_msg {
         serial => $serial,
     );
 
-    $self->{'_endian'} ||= 'le';
+    # Use native byte order by default.
+    $self->{'_endian'} ||= (pack 'n', 1) eq (pack 'l', 1) ? 'be' : 'le';
+
     $self->{'_to_str_fn'} ||= "to_string_$self->{'_endian'}";
 
     my ($buf_sr, $fds_ar) = $msg->can($self->{'_to_str_fn'})->($msg);
